#!/usr/bin/env python

import os, sys, string

if not hasattr(sys, 'version_info') or sys.version_info < (2,3,3,'final',0):
    raise SystemExit, "Python 2.3.3 or later required to build PyMidas."

from distutils.core import setup, Extension
from distutils.sysconfig import *
ver = sys.version_info
python_exec = 'python' + str(ver[0]) + '.' + str(ver[1])

py_includes = get_python_inc(plat_specific=1)
py_libs =  get_python_lib(plat_specific=1, standard_lib = 1)
py_extralibs = py_libs + '/config'

if os.uname()[ 0 ] == 'SunOS' :
	os.environ['CC'] = 'gcc -shared -O2 -g'
	os.environ['CXX']= 'gcc -shared -O2 -g' 
	os.environ['LDSHARED'] = 'gcc -shared -mimpure-text' 




def dolocal( ):
	"""Adds a command line options --midashome=<install-dir> and --midvers=<midas-ver> ."""

	if "--help" in sys.argv:
		print >> sys.stderr
		print >> sys.stderr, "Options:"
		print >> sys.stderr, "  --midashome=<MIDAS installation location>"
		print >> sys.stderr, "  --midvers=<MIDAS version>"
		print >> sys.stderr, "  --local=<install-dir>"
                

        
	midashome = ''
	midvers = ''
        # First take values if MIDASHOME or MIDVERS are set
        print 'Searching MIDAS environment variables...'
	if os.environ.has_key( 'MIDASHOME' ):
		midashome = os.environ[ 'MIDASHOME' ]
	if os.environ.has_key( 'MIDVERS' ):
		midvers = os.environ[ 'MIDVERS' ]		


        if midashome == '':
            print 'Not Found.'
            print 'Searching Midas from /usr/local...'
            # Second default loction /usr/local/bin        
            executable = '/usr/local/bin/inmidas'
            while os.path.islink(executable):
                executable = os.readlink(executable)
                midasDir = os.path.dirname(executable)
                midasinfo = midasDir.split('system')[0]
                midasinfo = midasinfo.split('midas')
                midashome = midasinfo[0] + 'midas'
                midvers = midasinfo[1].replace('/','')
                

        if midashome == '':
            print 'Not Found.'
            print 'Searching Midas from /scisoft...'
            # alternative  default loction /scisoft/bin        
            executable = '/scisoft/bin/inmidas'
            while os.path.islink(executable):
                executable = os.readlink(executable)
                midasDir = os.path.dirname(executable)
                midasinfo = midasDir.split('system')[0]
                midasinfo = midasinfo.split('midas')
                midashome = midasinfo[0] + 'midas'
                midvers = midasinfo[1].replace('/','')

        remargs=[]
	for a in sys.argv:
            if "--midashome=" in a:
                midashome=a.split( '=' )[ 1 ]
                remargs.append(a)		                
            if  "--midvers=" in a:
                midvers = a.split( '=' )[ 1 ]
                remargs.append(a)

            if "--local=" in a:
                dir = a.split("=")[1]
                sys.argv.extend([
                    "--install-lib="+dir,
                    "--install-headers="+os.path.join(dir,"pymidas"),
                    "--install-scripts="+os.path.join(dir,"pymidas"),
                    "--install-data="+os.path.join(dir,"pymidas")
                    ])
                remargs.append(a)
            
        for a in remargs:    
            sys.argv.remove(a)
        
        return midashome, midvers


def dosetup( midashome, midvers, data_dir ):
        #--------------------------------------------------
        # Some constants
	name_txt = 'PyMidas C-layer'
	version_txt = '0.95'

	ostype_linux   = '0'
	ostype_solaris = '0'
	ostype_macos   = '0'

        #--------------------------------------------------
        # Find the correct operating system
        #
	if os.uname()[ 0 ] == 'Linux' :
		ostype_linux = '1'
	if os.uname()[ 0 ] == 'SunOS' :
		ostype_solaris = '1'
	if os.uname()[ 0 ] == 'Darwin' :
		ostype_macos = '1'

	#if '--help' not in sys.argv:
		#print
		#print '*** Building',name_txt,'version',version_txt,'for',os.uname()[ 0 ],'***'
		#print
		#print 'MIDASHOME:' + midashome, 'MIDVERS:' + midvers

        if '--help' not in sys.argv:

            if midashome =='':
                print 'Not Found!'
                raise SystemExit, "Cannot install PyMidas. Midas not found"
            else:
                print 'Found.'
                print 'Midas location is',midashome
                print 'Midas version is', midvers
                if midvers =='':
                    raise SystemExit, "Cannot install PyMidas if MIDVERS is not set!"
	
                if midvers not in  ['05JUN','05SEPPRE','05SEP','05SEPpl1.0','05SEPpl1.1']:
                    print "Warning! MIDAS 05JUN or later is required to build PyMidas."
                    print "If you have older version, install a  new version or give location to  Midas"
                    print "with options --midashome=<location> --midvers=<midasversion>"
                    

                    string = str( raw_input( "Continue (y/n) ") )
                    if string[ 0 ] == 'n' or string[ 0 ] == 'N': 
                        raise SystemExit, "Exiting ..."

	midaspath = midashome + '/' + midvers

        if ostype_solaris =='1':
            libcurses ='curses'
        else:
            libcurses ='ncurses' 
        

        #--------------------------------------------------
        # Main defination for the module
        #
	module1 = Extension('pymidas/_pymidas_c_00',
  
			    define_macros = [('MAJOR_VERSION', '1'),
					     ('MINOR_VERSION', '0'),
					     ('OSTYPE_LINUX', ostype_linux),
					     ('OSTYPE_SOLARIS', ostype_solaris),
					     ('OSTYPE_MACOS', ostype_macos)],                                   
			    include_dirs = ['/usr/local/include', midaspath + '/incl', py_includes],
			    libraries = ['midas','subplot','plot','gmidas','disp','agen',
					 'dl','pthread','xcon','prep','gen','gcc',
					 'idicl','readline',libcurses,'m','st','stdc++', python_exec],
			    library_dirs = ['/usr/local/lib', py_libs+'/config',
					    midaspath + '/lib',
					    midaspath + '/monit'],
		    
			    sources = ['src/pymidas-layer1.cpp'
				       ]) 	

	setup (name=name_txt,
	       version=version_txt,
	       description = 'This is a PyMidas package', 
	       author = 'Tero Oittinen, Sami Maisala  / Sampo Project',
	       author_email = 'pymidas@astro.helsinki.fi',
	       url = 'http://www.eso.org/sampo/pymidas',
	       long_description = '''This is PyMidas/SAMPO package.''',
               platforms = ["unix"],
               packages = ['pymidas'],
               package_dir = {'pymidas':'lib'},
               scripts = ['lib/pymidas'],
	       ext_modules = [module1])



def getDataDir(args):
    for a in args:
        if string.find(a, '--home=') == 0:
            dir = string.split(a, '=')[1]
            data_dir = os.path.join(dir, 'lib/python/pymidas')
        elif string.find(a, '--prefix=') == 0:
            dir = string.split(a, '=')[1]
            data_dir = os.path.join(dir, 'lib', python_exec, 'site-packages/pymidas')
        elif a.startswith('--install-data='):
            dir = string.split(a, '=')[1]
            data_dir = dir
        else:
            data_dir = os.path.join(sys.prefix, 'lib', python_exec, 'site-packages/pymidas')
    return data_dir




def main():
    args = sys.argv
    midashome, midvers = dolocal()
    data_dir = getDataDir(args)
    dosetup( midashome, midvers , data_dir )



if __name__ == "__main__":
    main()


